#include <joyos.h>
//#include "DriveCode.h"

typedef struct RobotPosition
{
    int x;
    int y;
    int theta;
}RobotPosition;

typedef struct GoalPosition
{
    int x;
    int y;
}GoalPosition;

//DriveCode.h
void simpleDriveForward(int);
void simpleHeadingDrive(int, int);
void proportionalDriveForward(int, int);
float limit_vel(float); // to prevent motor speeds from exceeding limits
void stop(); //motor velocities set to 0
void recalibrateGyro(); //Stops robot, calibrates gyro with VPS
RobotPosition getCurrentPosition();
bool getColor();
void circleBlue();
void circleRed();
void rollerBlue();
void rollerRed();
void goToTerritory(int);
void goToRoller(bool);
bool goToLever();
void goToMiasmador(bool);
void readPosition();
int findClosestNotOurs();
int findClosestMine();
void holdBalls();
void releaseBalls();
RobotPosition findOpponent();
float getSimpleDistanceTo(int, int, int ,int);


#define LEFTMOTORPORT (1)
#define RIGHTMOTORPORT (0)
#define MAX_VEL (255)
#define THETA_UNITS_TO_DEGREES (0.08793)
//^DriveCode.h

//For vision system
extern volatile uint8_t robot_id;

bool teamBlue;

int usetup (void) {
    robot_id = 14;
    gyro_init(8, 1400000, 500L);
    copy_objects();
    gyro_set_degrees((game.coords[0].theta * THETA_UNITS_TO_DEGREES));

    teamBlue = getColor();
    return 0;

/*    //Gyroscope reset timer
    timeOfLastGoal = 0;
    lastGoalX = 0;
    lastGoalY = 0;
    return 0;*/
}

int umain (void)
{

    uint32_t startTime = get_time();
    holdBalls();

	//regular operations - written 1/27 by Stephanie

    if(teamBlue)
        circleBlue();
    else
        circleRed();

    recalibrateGyro();

    int mineCount = 0;
    while((get_time() - startTime)<100000) //get balls from 3 territories before dumping; interrupt built in
    {
        if (findClosestMine() == 7) //This only occurs when rate limit for all territories is greater than 15 seconds
        {
            goToTerritory(findClosestNotOurs());
            goToRoller(teamBlue);
        }
        else
        {
            goToTerritory(findClosestMine());
            goToRoller(teamBlue);
            if (goToLever())
                mineCount++;
            if ((mineCount%3 == 0) && (mineCount != 0))
                goToMiasmador(teamBlue);
        }
    }

    if (mineCount%3 != 0)
        goToMiasmador(teamBlue);

	while (get_time() >= 100000)
	{
		goToTerritory(findClosestNotOurs());
		goToRoller(teamBlue);
    }

    return 0;
}
